const St = imports.gi.St;
const Lang = imports.lang;
const PanelMenu = imports.ui.panelMenu;
const PopupMenu = imports.ui.popupMenu;
const Main = imports.ui.main;
const GLib = imports.gi.GLib;
const Util = imports.misc.util;
const Mainloop = imports.mainloop;
const Extension = imports.misc.extensionUtils.getCurrentExtension();
const Clutter = imports.gi.Clutter;
const Gio = imports.gi.Gio;

function amdTemperature() {
    this._init.apply(this, arguments);
}

amdTemperature.prototype = {
    __proto__: PanelMenu.Button.prototype,

    _init: function(){
        PanelMenu.Button.prototype._init.call(this, 0, 'amdtemp');

        // Set Logo
        var theme = imports.gi.Gtk.IconTheme.get_default();
        let icon_dir = Extension.dir.get_child('icons');
        theme.append_search_path(icon_dir.get_path());
		let gicon=Gio.icon_new_for_string(Extension.path + "/icons/amd-logo-32-symbolic.png");
        this._logo = new St.Icon({ gicon: gicon,
									style_class: 'system-status-icon'});

        this.lang = {
            'gpu' : 'AMD GPU Core Temp'
        };
        this.statusLabel = new St.Label({
            y_align: Clutter.ActorAlign.CENTER,
            text: "--",
            style_class: "temperature-label"
        });

		 // destroy all previously created children, and add our statusLabel
        this.actor.get_children().forEach(function(c) {
            c.destroy()
        });

		// create a new box layout, composed of a) a "bin", b) the label
        let box = new St.BoxLayout({ name: 'tempBox' });
        this.actor.add_actor(box);
        box.add_actor(this._logo);
        box.add_actor(this.statusLabel);




		this.sensorsPath = this._detectSensors();
        this.command=["xdg-open", "http://zerosubnet.com"];
    		if(this.sensorsPath){
				if(this.sensorsPath !== '/usr/bin/aticonfig'){
					this.title='Error';
					log("could not find:#/usr/bin/aticonfig - strange :S");
                	this.content='Could not find aticonfig - Please install AMD Catalyst Control Center to run the extension properly.';
				}else { this.title='Success';
                		this.content='ATIConfig found.';
				}
			}
        //},

        this._update_temp();
        // update every 15 seconds
        event = GLib.timeout_add_seconds(0, 15, Lang.bind(this, function () {
            this._update_temp();
            return true;
        }));
    },



	_detectSensors: function(){
        // detect if amdcccle or aticonfig is installed
        let ret = GLib.find_program_in_path("aticonfig");
        return ret; // path to AMD tools program or empty string.
		
    },




	_update_temp: function() {
        let items = new Array();
        let tempInfo=null;
        if (this.sensorsPath){
            let sensors_output = '';
            if (this.sensorsPath.substr(9) == 'aticonfig') {
                sensors_output = GLib.spawn_command_line_sync(this.sensorsPath + " --adapter=0 --odgt"); // get the output of the aticonfig command.
            }
		
            if(sensors_output[0]) tempInfo = this._findTemperatureFromSensorsOutput(sensors_output[1].toString()); // get temp from either command output.
			
            // Parse all temperature values and save them.
            if (tempInfo){
                // destroy all items in popup
                this.menu.box.get_children().forEach(function(c) {
                    c.destroy()
                });
                var s=0, n=0; // sum and count
                for (let adapter in tempInfo){
                    if(adapter!=0){
                        if (tempInfo[adapter]['temp']>0){
                            s+=tempInfo[adapter]['temp'];
                            n++;
                            items.push(this.lang[adapter] + ' : '+this._formatTemp(tempInfo[adapter]['temp']));
                        }
                    }
                }
                if (n!=0) { // if temperature is detected
                    this.title=this._formatTemp(s/n); // set title as average
                }
                else if (this.sensorsPath.substr(1)=='Temperature -') { // Try to detect if card is on or not.
                    this.title='Off';
                    items.push('GPU temperature will be shown when the card is active.');
                }
            }
        }


		this.statusLabel.set_text(this.title);
        this.menu.box.get_children().forEach(function(c) {
            c.destroy()
        });
        let section = new PopupMenu.PopupMenuSection("GPU Temperature");
        if (items.length>0){
            let item;
            for each (let itemText in items){
                item = new PopupMenu.PopupMenuItem("", { reactive: false });
                item.actor.add_child(new St.Label({
                    text:itemText,
                    style_class: "sm-label"
                }));
                section.addMenuItem(item);
            }
        }else{ // or open a browser to report the issue.
            let command=this.command;
            let item;
            item = new PopupMenu.PopupMenuItem("");
            item.actor.add_child(new St.Label({
                text:this.content,
                style_class: "sm-label"
            }));
            item.connect('activate',function() {
                Util.spawn(command);
            });
            section.addMenuItem(item);
        }
        this.menu.addMenuItem(section);
    },

	createSectionForText: function(txt){
		let section = new PopupMenu.PopupMenuSection("GPU Temperature");
		let item = new PopupMenu.PopupMenuItem("");
		item.addActor(new St.Label({
			text:txt,
			style_class: "sm-label"
		}));
		section.addMenuItem(item);
		return section;
	},






	_findTemperatureFromSensorsOutput: function(txt){
        let senses_lines=txt.split("\n");
        let line = '';
        let type = '';
        let s= new Array();
        let n=0,c=0;
        let f;
        // iterate through each lines
        for(let i = 0; i < senses_lines.length; i++) {
            line = senses_lines[i];
            if(senses_lines[i]){
                let amdtool = this.sensorsPath.substr(9);
                switch (amdtool) {
                    case 'settings':
                        s['gpu'] = new Array();
                        s['gpu']['temp']=parseFloat(senses_lines[i].substr(0,2));
                        c++;
                    case 'smi':
                        // remove all space characters
                        senses_lines[i]=senses_lines[i].replace(/\s/g, "");
                        if(senses_lines[i]=='Temperature'){
                            s['gpu'] = new Array();
                            s['gpu']['temp']=parseFloat(senses_lines[i+1].substr(-4,2));
                            c++;
						}
					case 'aticonfig':
                        // remove all space characters
                        senses_lines[i]=senses_lines[i].replace(/\s/g, "");
                        if(senses_lines[i].substr(0,20)=='Sensor0:Temperature-'){
                            s['gpu'] = new Array();
                            s['gpu']['temp']=parseFloat(senses_lines[i].substr(20,2));
                            c++;
						}
                        
                    default:
                        break;
                }
            }
        }
        return s;
    },


	_formatTemp: function(t) {
        // uncomment the next line to display temperature in Fahrenheit
        // return this._toFahrenheit(t).toString()+"\u00b0F";
        return (Math.round(t*10)/10).toFixed(0).toString()+"\u00b0C";
    }
}

function debug(a){
    global.log(a);
    Util.spawn(['echo',a]);
}


function init() {
// do nothing
}

let indicator;
let event=null;

function enable() {
    indicator = new amdTemperature();
    Main.panel.addToStatusArea('amdtemp', indicator);
}

function disable() {
    indicator.destroy();
    Mainloop.source_remove(event);
    indicator = null;
}




